# Generated by Django 4.0.6 on 2022-09-17 11:35

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aadiscordbot', '0006_goodbyemessage_welcomemessage'),
    ]

    operations = [
        migrations.CreateModel(
            name='QuoteMessage',
            fields=[
                ('message', models.BigIntegerField(
                    primary_key=True, serialize=False)),
                ('content', models.CharField(max_length=1000)),
                ('datetime', models.DateTimeField(blank=True, null=True)),
                ('author', models.PositiveBigIntegerField()),
                ('author_nick', models.CharField(
                    blank=True, max_length=50, null=True)),
                ('reference', models.CharField(
                    help_text='Nickname for this quote', max_length=100)),
                ('channel', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='aadiscordbot.channels')),
                ('server', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='aadiscordbot.servers')),
            ],
        ),
    ]



# Generated by Django 4.0.10 on 2023-06-30 11:20

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aadiscordbot', '0010_alter_discordbot_options_ticketgroups'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='ticketgroups',
            options={'default_permissions': (), 'verbose_name': 'Ticket Cog Configuration',
                     'verbose_name_plural': 'Ticket Cog Configuration'},
        ),
        migrations.AddField(
            model_name='ticketgroups',
            name='ticket_channel',
            field=models.ForeignKey(
                default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, to='aadiscordbot.channels'),
        ),
    ]



"""
"Welcome" cog for discordbot - https://github.com/pvyParts/allianceauth-discordbot
"""
import asyncio
import logging

from discord.ext import commands

from allianceauth.services.modules.discord.models import DiscordUser

from aadiscordbot.models import GoodbyeMessage, WelcomeMessage
from aadiscordbot.app_settings import get_site_url

logger = logging.getLogger(__name__)


class Welcome(commands.Cog):
    """
    Responds to on_member_join events from discord
    """

    def __init__(self, bot):
        self.bot = bot

    @commands.Cog.listener("on_member_join")
    async def on_member_join(self, member):
        channel = member.guild.system_channel
        if channel is not None:
            try:
                # Give AA a chance to save the UID for a joiner.
                await asyncio.sleep(1)
                authenticated = DiscordUser.objects.get(
                    uid=member.id).user.has_perm("discord.access_discord")
            except Exception:
                authenticated = False
            if authenticated:
                try:
                    message = WelcomeMessage.objects.filter(
                        authenticated=True).order_by('?').first().message
                    message_formatted = message.format(
                        user_mention=member.mention,
                        guild_name=member.guild.name,
                        auth_url=get_site_url(),)
                    await channel.send(message_formatted)

                except IndexError as e:
                    logger.error(
                        'No Welcome Message configured for Discordbot Welcome cog')
                except Exception as e:
                    logger.error(e)

            else:
                try:
                    message = WelcomeMessage.objects.filter(
                        unauthenticated=True).order_by('?').first().message
                    message_formatted = message.format(
                        user_mention=member.mention,
                        guild_name=member.guild.name,
                        auth_url=get_site_url(),)
                    await channel.send(message_formatted)
                except Exception as e:
                    logger.error(e)


class Goodbye(commands.Cog):
    """
    Responds to on_member_remove events from discord
    """

    def __init__(self, bot):
        self.bot = bot

    @commands.Cog.listener("on_member_remove")
    async def on_member_remove(self, member):
        channel = member.guild.system_channel
        if channel is not None:
            try:
                # Give AA a chance to save the UID for a joiner.
                await asyncio.sleep(1)
                authenticated = DiscordUser.objects.get(
                    uid=member.id).user.has_perm("discord.access_discord")
            except Exception:
                authenticated = False
            if authenticated:
                # Authenticated
                try:
                    message = GoodbyeMessage.objects.filter(
                        authenticated=True).order_by('?').first().message
                    message_formatted = message.format(
                        user_mention=member.mention,
                        guild_name=member.guild.name,
                        auth_url=get_site_url(),)
                    await channel.send(message_formatted)
                except IndexError as e:
                    logger.error(
                        'No Welcome Message configured for Discordbot Goodbye cog')
                except Exception as e:
                    logger.error(e)

            else:
                # Un-Authenticated
                try:
                    message = GoodbyeMessage.objects.filter(
                        unauthenticated=True).order_by('?').first().message
                    message_formatted = message.format(
                        user_mention=member.mention,
                        guild_name=member.guild.name,
                        auth_url=get_site_url(),)
                    await channel.send(message_formatted)
                except Exception as e:
                    logger.error(e)


def setup(bot):
    """
    setup the cog
    :param bot:
    """
    if bot.get_cog("Welcome") is None:
        bot.add_cog(Welcome(bot))

    if bot.get_cog("Goodbye") is None:
        bot.add_cog(Goodbye(bot))



import logging

from discord.commands import SlashCommandGroup
from discord.ext import commands

from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist

from aadiscordbot import __branch__, __version__
from aadiscordbot.cogs.utils.decorators import sender_is_admin
from aadiscordbot.models import Channels, Servers

logger = logging.getLogger(__name__)


class Models(commands.Cog):
    """
    Django Model Population and Maintenance
    """

    def __init__(self, bot):
        self.bot = bot

    model_commands = SlashCommandGroup("models", "Django Model Population", guild_ids=[
                                       int(settings.DISCORD_GUILD_ID)])

    @model_commands.command(name="populate", guild_ids=[int(settings.DISCORD_GUILD_ID)])
    @sender_is_admin()
    async def populate_models(self, ctx):
        """
        Populates Django Models for every Channel in the Guild
        """
        await ctx.respond(f"Populating Models, this might take a while on large servers", ephemeral=True)
        try:
            Servers.objects.update_or_create(
                server=ctx.guild.id,
                defaults={"name": ctx.guild.name}
            )
        except Exception as e:
            logger.error(e)
        server = Servers.objects.get(server=ctx.guild.id)
        for channel in ctx.guild.channels:
            try:
                Channels.objects.update_or_create(
                    channel=channel.id,
                    defaults={
                        "name": channel.name,
                        "server": server
                    }
                )
            except Exception as e:
                logger.error(e)

        return await ctx.respond(f"Django Models Populated for {ctx.guild.name}", ephemeral=True)

    @commands.Cog.listener("on_guild_channel_delete")
    async def on_guild_channel_delete(self, channel):
        try:
            deleted_channel = Channels.objects.get(channel=channel.id)
            deleted_channel.deleted = True
            deleted_channel.save()
        except ObjectDoesNotExist:
            #this is fine
            pass
        except Exception as e:
            logger.error(e)

    @commands.Cog.listener("on_guild_channel_create")
    async def on_guild_channel_create(self, channel):
        try:
            Channels.objects.create(
                channel=channel.id,
                name=channel.name,
                server=Servers.objects.get(server=channel.guild.id)
            )
        except Exception as e:
            logger.error(e)

    @commands.Cog.listener("on_guild_channel_update")
    async def on_guild_channel_update(self, before_channel, after_channel):
        if before_channel.name == after_channel.name:
            pass
        else:
            try:
                Channels.objects.update_or_create(
                    channel=after_channel.id,
                    defaults={"name": after_channel.name}
                )
            except Exception as e:
                logger.error(e)

    @commands.Cog.listener("on_guild_update")
    async def on_guild_update(self, before_guild, after_guild):
        if before_guild.name == after_guild.name:
            pass
        else:
            try:
                Servers.objects.update_or_create(
                    server=after_guild.id,
                    defaults={"name": after_guild.name}
                )
            except Exception as e:
                logger.error(e)


def setup(bot):
    bot.add_cog(Models(bot))



import logging
import sys
import time
import traceback
from datetime import datetime
from socket import timeout
from typing import Dict

import aiohttp
import discord
import pendulum
from celery.utils.time import rate
from discord import ApplicationContext, DiscordException
from discord.ext import commands, tasks
from kombu import Connection, Consumer, Queue
from kombu.utils.limits import TokenBucket
from redis import asyncio as aioredis

import django
import django.db
from django.conf import settings
from django.utils import timezone

from allianceauth import hooks

import aadiscordbot
from aadiscordbot import app_settings
from aadiscordbot.app_settings import (
    DISCORD_BOT_ACCESS_DENIED_REACT, DISCORD_BOT_MESSAGE_INTENT,
    DISCORD_BOT_PREFIX,
)
from aadiscordbot.cogs.utils.exceptions import NotAuthenticated

from . import bot_tasks
from .cogs.utils import context

description = """
AuthBot is watching...
"""

logger = logging.getLogger(__name__)

INVITE_URL = f"https://discord.com/api/oauth2/authorize?client_id={app_settings.DISCORD_APP_ID}&permissions=8&scope=bot%20applications.commands"

queuename = "aadiscordbot"
queue_keys = [f"{queuename}",
              f"{queuename}\x06\x161",
              f"{queuename}\x06\x162",
              f"{queuename}\x06\x163",
              f"{queuename}\x06\x164",
              f"{queuename}\x06\x165",
              f"{queuename}\x06\x166",
              f"{queuename}\x06\x167",
              f"{queuename}\x06\x168",
              f"{queuename}\x06\x169"]


class PendingQueue:
    def __init__(self):
        self.data = []

    def append(self, data):
        self.data.append(data)
        self.data.sort(key=lambda a: a[0])

    def pop_next(self):
        next_task = next(
            (x for x in self.data if x[0] < timezone.now()), False)
        if next_task:
            del (self.data[self.data.index(next_task)])
        return next_task

    def outstanding(self):
        return len(self.data)


class RateLimiter:
    def __init__(self):
        self.rate_buckets = {}

    def bucket_for_task(self, task):
        limit = rate(
            app_settings.DISCORD_BOT_TASK_RATE_LIMITS.get(task, "100/s"))
        return TokenBucket(limit, capacity=1)

    def check_rate_limit(self, task):
        if task not in self.rate_buckets:
            self.rate_buckets[task] = self.bucket_for_task(task)
        #logger.debug(f" {self.rate_buckets[task].capacity} {self.rate_buckets[task].fill_rate} {self.rate_buckets[task].expected_time()}")
        return self.rate_buckets[task].can_consume()

    def to_string(self):
        """
            Print of Task Limiter Stats
        """
        out = ["```"]
        for t, d in self.rate_buckets.items():
            out.append(
                f"{t}\n   Rate:  {d.fill_rate:.2}\n   TTL:   {d.expected_time():.2}")
        out.append("```")
        return "\n".join(out)


class Statistics:
    def __init__(self):
        self.task_buckets = {}
        self.start_time = timezone.now()

    def add_task(self, task):
        date = timezone.now().strftime("%Y/%m/%d")
        if date not in self.task_buckets:
            self.task_buckets[date] = {}
        if task not in self.task_buckets[date]:
            self.task_buckets[date][task] = 0
        self.task_buckets[date][task] += 1

    def reset(self):
        self.task_buckets = {}

    def to_string(self):
        """
            Print of Run Task Stats
        """
        gap = "                                         "
        hdr = "  Task                                   Count"
        out = ["```", "Date"]
        for d, ts in self.task_buckets.items():
            out.append(f"{d}")
            out.append(hdr)
            for t, c in ts.items():
                out.append(f"    {t}{gap[len(str(t)):41]}{c}")
            out.append("\n")
        out.append("```")
        return "\n".join(out)


class AuthBot(commands.Bot):
    def __init__(self):
        django.setup()
        client_id = app_settings.DISCORD_APP_ID
        intents = discord.Intents.default()
        intents.members = True
        intents.message_content = app_settings.DISCORD_BOT_MESSAGE_INTENT

        super().__init__(
            command_prefix=DISCORD_BOT_PREFIX,
            description=description,
            intents=intents,
        )
        print(f"Authbot Started with command prefix {DISCORD_BOT_PREFIX}")

        self.redis = None
        self.redis = self.loop.run_until_complete(aioredis.from_url(getattr(
            settings, "BROKER_URL", "redis://localhost:6379/0"), encoding="utf-8", decode_responses=True))
        print('redis pool started', self.redis)
        self.client_id = client_id
        self.session = aiohttp.ClientSession(loop=self.loop)

        self.tasks = []
        self.pending_tasks = PendingQueue()
        self.rate_limits = RateLimiter()
        self.statistics = Statistics()

        self.message_connection = Connection(
            getattr(settings, "BROKER_URL", 'redis://localhost:6379/0'))

        queues = []
        for que in queue_keys:
            queues.append(Queue(que))

        self.message_consumer = Consumer(self.message_connection, queues, callbacks=[
                                         self.on_queue_message], accept=['json'])

        self.cog_names_loaded = []
        self.cog_names_failed = []
        hooks_ = hooks.get_hooks("discord_cogs_hook")
        for hook in hooks_:
            for cog in hook():
                try:
                    self.load_extension(cog)
                    self.cog_names_loaded.append(cog)
                except Exception as e:
                    logger.exception(f"Failed to load cog {cog}")
                    self.cog_names_failed.append(cog)

    def on_queue_message(self, body, message):
        logger.debug(f'RECEIVED MESSAGE: {body!r}')
        try:
            task_headers = message.headers
            eta = task_headers.get("eta", False)
            _args = body[0]
            _kwargs = body[1]

            if 'aadiscordbot.tasks.' in task_headers["task"]:
                task = task_headers["task"].replace("aadiscordbot.tasks.", '')
                task_function = getattr(bot_tasks, task, False)
                if task_function:
                    if eta:
                        logger.debug(f"ETA RECEIVED: {eta}")
                        eta_date = datetime.fromisoformat(eta)
                        if eta_date < timezone.now():
                            self.tasks.append((task_function, _args, _kwargs))
                        else:
                            self.pending_tasks.append(
                                (eta_date, (task_function, _args, _kwargs)))
                    else:
                        self.tasks.append((task_function, _args, _kwargs))
                    if not bot_tasks.run_tasks.is_running():
                        bot_tasks.run_tasks.start(self)
                else:
                    logger.debug("No bot_task for that auth_task?")
            else:
                logger.debug("i got an invalid auth_task")

        except Exception as e:
            logger.error("Queue Consumer Failed")
            logger.error(e, exc_info=1)
        message.ack()

    async def on_ready(self):
        if not hasattr(self, "currentuptime"):
            self.currentuptime = pendulum.now(tz="UTC")
        if not hasattr(self, "statistics"):
            self.statistics.start_time = timezone.now()
        activity = discord.Activity(name="Everything!",
                                    application_id=0,
                                    type=discord.ActivityType.watching,
                                    state="Monitoring",
                                    details="Waiting for Shenanigans!",
                                    emoji={"name": ":smiling_imp:"}
                                    )
        await self.change_presence(activity=activity)

        self.poll_queue.start()
        logger.info("Ready")

    async def close(self):
        # return tasks to queue

        # recursive import
        from aadiscordbot import tasks as celery_tasks

        self.poll_queue.stop()
        bot_tasks.run_tasks.stop()

        recovered_messages = 0
        lost_messages = 0
        for task in self.tasks:
            task_function = getattr(celery_tasks, task[0].__name__, False)
            if task_function:
                recovered_messages += 1
                task_function.apply_async(args=task[1], kwargs=task[2])
            else:
                lost_messages += 1

        for task in self.pending_tasks.data:
            task_function = getattr(celery_tasks, task[1][0].__name__, False)
            if task_function:
                recovered_messages += 1
                task_function.apply_async(
                    args=task[1][1], kwargs=task[1][2], eta=task[0])
            else:
                lost_messages += 1

        if recovered_messages:
            logger.info(
                f"Returned {recovered_messages} message pending tasks to the celery queue")
        if lost_messages:
            logger.warning(
                f"Lost {recovered_messages} message pending tasks while trying to return them to the celery queue")

        await super().close()

    async def process_commands(self, message):
        ctx = await self.get_context(message)
        if ctx.command is None:
            return
        django.db.close_old_connections()
        await self.invoke(ctx)
        django.db.close_old_connections()

    async def on_interaction(self, interaction):
        try:
            django.db.close_old_connections()
            await self.process_application_commands(interaction)
            django.db.close_old_connections()
        except Exception as e:
            logger.error("Interaction Failed {e}", stack_info=True)

    async def on_message(self, message):
        if message.author.bot:
            return
        await self.process_commands(message)

    async def sync_commands(self, *args, **kwargs):
        try:
            return await super(__class__, self).sync_commands(*args, **kwargs)
        except discord.Forbidden:
            logger.error(
                "******************************************************")
            logger.error("|   AuthBot was Unable to Sync Slash Commands!!!!")
            logger.error(
                "|   Please ensure your bot was invited to the server with the correct scopes")
            logger.error("|")
            logger.error("|   To redo your scopes,")
            logger.error("|      1. Refresh Scopes with this link:")
            logger.error(f"|        {INVITE_URL}   ")
            logger.error(
                "|      2. Move the bots role to top of the roles tree if its not there already")
            logger.error("|      3. Restart Bot")
            logger.error(
                "******************************************************")

    @tasks.loop(seconds=1.0)
    async def poll_queue(self):
        message_avail = True
        while message_avail:
            try:
                with self.message_consumer:
                    self.message_connection.drain_events(timeout=0.01)
            except timeout as e:
                # logging.exception(e)
                message_avail = False

        next_task = self.pending_tasks.pop_next()
        while next_task:
            self.tasks.append(next_task[1])

            if not bot_tasks.run_tasks.is_running():
                bot_tasks.run_tasks.start(self)

            next_task = self.pending_tasks.pop_next()

    async def on_resumed(self):
        print("Resumed...")

    async def on_command_error(self, ctx, error):
        if isinstance(error, commands.BadArgument):
            logger.error(error)
            await ctx.send(error)
        elif isinstance(error, commands.MissingRequiredArgument):
            logger.error(error)
            await ctx.send(error)
        elif isinstance(error, commands.NoPrivateMessage):
            logger.error(error)
            await ctx.send(error)
        elif isinstance(error, commands.CommandInvokeError):
            logger.error(error)
            return await ctx.send(error)
        elif isinstance(error, commands.BotMissingPermissions):
            await ctx.send(
                "Sorry, I don't have the required permissions to do that here:\n{}".format(
                    error.missing_permissions)
            )
        elif isinstance(error, commands.MissingPermissions):
            await ctx.message.add_reaction(chr(DISCORD_BOT_ACCESS_DENIED_REACT))
            await ctx.message.reply("Sorry, you do not have permission to do that here.")
        elif isinstance(error, commands.NotOwner):
            logger.error(error)
            await ctx.send(error)
        elif isinstance(error, commands.CommandOnCooldown):
            await ctx.message.add_reaction(chr(0x274C))
        elif isinstance(error, commands.CheckFailure):
            await ctx.send(error)
        else:
            logger.error(f"Unknown Error {error}", exc_info=True)
            await ctx.send("Something Went Wrong, Please try again Later.",)

    async def on_application_command_error(self, context: ApplicationContext, exception: DiscordException) -> None:
        if isinstance(exception, commands.CheckFailure):
            await context.send_response(exception, ephemeral=True)
        elif isinstance(exception, commands.MissingPermissions):
            await context.send_response(exception, ephemeral=True)
        elif isinstance(exception, NotAuthenticated):
            await context.send_response(exception, ephemeral=True)
        else:  # Catch everything, and close out the interactions gracefully.
            logger.error(f"Unknown Error {exception}")
            logger.error("\n".join(traceback.format_tb(
                exception.original.__traceback__)))

            if app_settings.DISCORD_BOT_SEND_FAILURE_MESSAGES and app_settings.DISCORD_BOT_FAILURE_MESSAGES_CHANNEL:
                message = [f"`{context.command}` failed for `{context.author}`\n",
                           f"**Exception:** ```{exception}```",
                           f"\n**Interaction:** ```{context.interaction.data}```",
                           f"\n**Trace:**```"]
                message += traceback.format_tb(
                    exception.original.__traceback__)
                message.append("\n```")
                from . import tasks
                tasks.send_message(message="\n".join(message),
                                   channel_id=app_settings.DISCORD_BOT_FAILURE_MESSAGES_CHANNEL)
            await context.respond("Something Went Wrong, Please try again Later.", ephemeral=True)

    def run(self):
        # self.load_extension("aadiscordbot.slash.admin")
        try:

            logger.info(
                "******************************************************")
            logger.info(f"         ##            Alliance Auth 'AuthBot'")
            logger.info(
                f"        ####           Version         :  {aadiscordbot.__version__}")
            logger.info(
                f"       #######         Branch          :  {aadiscordbot.__branch__}")
            logger.info(
                f"      #########        Message Intents :  {app_settings.DISCORD_BOT_MESSAGE_INTENT}")
            logger.info(
                f"     ######## ((       Prefix          :  {app_settings.DISCORD_BOT_PREFIX}")
            logger.info(
                f"    ###### ((((((      Bot Join Link   :  {INVITE_URL}")
            logger.info(f"   ###        ((((     Starting up...")
            logger.info(f"  ##             ((")
            logger.info(f"                       [Cogs Loaded]")
            for c in self.cog_names_loaded:
                logger.info(f"                         - {c}")
            if len(self.cog_names_failed):
                logger.info(f"                       [Cog Failures]")
                for c in self.cog_names_failed:
                    logger.info(f"                         - {c}")
            logger.info(
                "******************************************************")
            super().run(app_settings.DISCORD_BOT_TOKEN, reconnect=True)
        except discord.PrivilegedIntentsRequired as e:
            logger.error("Unable to start bot with Messages Intent! Going to Sleep for 2min. "
                         "Please enable the Message Intent for your bot. "
                         "https://support-dev.discord.com/hc/en-us/articles/4404772028055"
                         f"{e}", exc_info=False)
            logger.info("If you wish to run without the Message Intent disable it in the local.py. "
                        "DISCORD_BOT_MESSAGE_INTENT=False", exc_info=False)
            time.sleep(120)
        except Exception as e:
            logger.error("Unable to start bot! going to sleep for 2 min. "
                         f"{e}", exc_info=True)
            time.sleep(120)



# Generated by Django 3.2.5 on 2021-08-01 06:41

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
        ('aadiscordbot', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ReactionRoleMessage',
            fields=[
                ('message', models.BigIntegerField(
                    primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=100)),
            ],
            options={
                'permissions': (('manage_reactions', 'Can Manage Reaction Roles'),),
            },
        ),
        migrations.CreateModel(
            name='ReactionRoleBinding',
            fields=[
                ('id', models.AutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('emoji', models.CharField(max_length=100)),
                ('emoji_text', models.CharField(max_length=100)),
                ('group', models.ForeignKey(default=None, null=True,
                 on_delete=django.db.models.deletion.CASCADE, to='auth.group')),
                ('message', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='aadiscordbot.reactionrolemessage')),
            ],
        ),
    ]



import logging
from random import randrange

from discord.ext import commands

from django.conf import settings

from aadiscordbot import __branch__, __version__

logger = logging.getLogger(__name__)


class Abuse(commands.Cog):
    """
    All about me!
    """

    def __init__(self, bot):
        self.bot = bot

    @commands.Cog.listener("on_message")
    async def respond_to_abuse(self, message):
        replies = [
            "get a dog up ya, cunt",
            "Can I help you? Do you want a time out?",
            "No u",
            "How about Nooooooooo!",
            "Thats a horrible thing to say about your mother",
            "I know you are, but what am I",
            "I'm sorry, did you say something? I don't speak moron",
            "Command accepted! Now commencing with your time out",
            "You kiss your mother with that mouth?!?",
            "Thats what she said",
            "You're so ugly, you scared the crap out of the toilet.",
            "It's better to let someone think you are an Idiot than to open your mouth and prove it.",
            "Your birth certificate is an apology letter from the condom factory.",
            "The only way you'll ever get laid is if you crawl up a chicken's ass and wait.",
            "You're so fake, Barbie is jealous.",
            "I’m jealous of people that don’t know you!",
            "Brains aren't everything. In your case they're nothing.",
            "I don't know what makes you so stupid, but it really works.",
            "I can explain it to you, but I can’t understand it for you.",
            "Roses are red violets are blue, God made me pretty, what happened to you?",
            "Calling you an idiot would be an insult to all the stupid people.",
            "You, sir, are an oxygen thief!",
            "I'd slap you, but that would be animal abuse.",
            "They say opposites attract. I hope you meet someone who is good-looking, intelligent, and cultured.",
            "Stop trying to be a smart ass, you're just an ass.",
            "I'm busy now. Can I ignore you some other time?",
            "Why don't you slip into something more comfortable... like a coma.",
            "To make you laugh on Saturday, I need to you joke on Wednesday.",
            "Pardon me, but you've obviously mistaken me for someone who gives a damn.",
            "https://media.tenor.com/x8v1oNUOmg4AAAAC/rickroll-roll.gif"
        ]
        rand = randrange(0, len(replies)-1)
        if message.mention_everyone:
            return
        try:
            if self.bot.user.mentioned_in(message):
                if message.author.id == 318309023478972417:
                    await message.reply("https://media.tenor.com/x8v1oNUOmg4AAAAC/rickroll-roll.gif")
                else:
                    await message.reply(replies[rand])
        except:
            pass


def setup(bot):
    bot.add_cog(Abuse(bot))



# Generated by Django 4.0.10 on 2023-06-30 10:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
        ('aadiscordbot', '0009_alter_quotemessage_options'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='discordbot',
            options={'default_permissions': (), 'managed': False, 'permissions': (
                ('basic_access', 'Can access this app.'), ('member_command_access', 'can access the member commands.'))},
        ),
        migrations.CreateModel(
            name='TicketGroups',
            fields=[
                ('id', models.AutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('groups', models.ManyToManyField(blank=True,
                 help_text='Pingable groups for ticketing', to='auth.group')),
            ],
            options={
                'default_permissions': (),
            },
        ),
    ]



import importlib
import logging
import warnings
from datetime import timedelta

from discord import Embed, Member
from discord.ext import tasks
from discord.ext.commands import Bot
from discord.ui import View

import django
from django.conf import settings
from django.utils import timezone

logger = logging.getLogger(__name__)

GUILD_ID = settings.DISCORD_GUILD_ID


@tasks.loop()
async def run_tasks(bot: Bot):
    django.db.close_old_connections()

    if len(bot.tasks) > 0:
        task, args, kwargs = bot.tasks.pop(0)
        requeue_task = False
        if hasattr(bot, 'rate_limits'):
            if not bot.rate_limits.check_rate_limit(task.__name__):
                requeue_task = True
        if requeue_task:
            # timeout ( for now lets just add 1 second )
            timeout = 1
            eta = timezone.now() + timedelta(seconds=timeout)
            bot.pending_tasks.append((eta, (task, args, kwargs)))
            #logger.debug(f"Rate Limit hit! Re Queueing `{task}`")
        else:
            try:
                await task(bot, *args, **kwargs)
                bot.statistics.add_task(task.__name__)
                bot.dispatch("authbot_task_completed", task.__name__)
            except Exception as e:
                bot.dispatch("authbot_task_failed",
                             task.__name__, args, kwargs, e)
                logger.error(
                    f"Failed to run task {task} {args} {kwargs} {e}", exc_info=True)
    else:
        run_tasks.stop()
    django.db.close_old_connections()


async def send_channel_message_by_discord_id(bot, channel_id, message, embed=False, view_class=False, view_args=[], view_kwargs={}):
    logger.debug(f"Sending Channel Message to Discord ID {channel_id}")
    e = None
    v = None
    if embed:
        e = Embed.from_dict(embed)

    if view_class:
        m = ".".join(view_class.split(".")[:-1])
        c = view_class.split(".")[-1]
        my_module = importlib.import_module(m)
        Viewclass = getattr(my_module, c)
        v = Viewclass(*view_args, **view_kwargs)

    await bot.get_channel(channel_id).send(message, embed=e, view=v)


async def send_channel_message(bot, channel_id, message, embed=False):
    warnings.warn(
        "send_channel_message is deprecated, use send_channel_message_by_discord_id instead",
        DeprecationWarning
    )
    await send_channel_message_by_discord_id(bot, channel_id, message, embed=embed)


async def send_direct_message_by_discord_id(bot, discord_user_id, message, embed=False):
    logger.debug(f"Sending DM to Discord ID {discord_user_id}")

    channel = await bot.get_user(discord_user_id).create_dm()
    if embed:
        e = Embed.from_dict(embed)
        await channel.send(message, embed=e)
    else:
        await channel.send(message)


async def send_direct_message(bot, discord_user_id, message, embed=False):
    warnings.warn(
        "send_direct_message is deprecated, use send_direct_message_by_discord_id instead",
        DeprecationWarning
    )
    await send_direct_message_by_discord_id(bot, discord_user_id, message, embed=embed)


async def send_direct_message_by_user_id(bot, user_pk, message, embed=False):
    # App isn't loaded when this file is imported, so importing here
    from django.contrib.auth.models import User
    logger.debug(f"Sending DM to User ID {user_pk}")
    user = User.objects.get(pk=user_pk)
    if hasattr(user, "discord"):
        discord_user_id = user.discord.uid
        channel = await bot.get_user(discord_user_id).create_dm()
        if embed:
            e = Embed.from_dict(embed)
            await channel.send(message, embed=e)
        else:
            await channel.send(message)
    else:
        logger.debug(f"No discord account on record for user_pk={user_pk}")


async def pop_user_group_cache(bot, user_pk):
    logger.debug(f"Refreshing user cache {user_pk}")
    user = bot.get_guild(int(GUILD_ID)).get_member(user_pk)
    r = user.roles[-1]
    await user.remove_roles(r)
    await user.add_roles(r)
    logger.info(
        f"Removed and added '{r}' to {user} to try and bust the invalid cache")



# Cog Stuff
import logging
from typing import Optional

import discord
from discord import ChannelType, Embed, command, ui
from discord.ext import commands

# AA Contexts
from django.conf import settings
from django.utils import timezone

from .. import models

logger = logging.getLogger(__name__)


def get_groups():
    groups = models.TicketGroups.get_solo().groups.all().order_by('name')
    out = []
    for g in groups:
        out.append(
            discord.SelectOption(
                label=f"{g.name}"
            )
        )
    return out


class HelpView(ui.View):
    """
        View for picking a group to assign a help thread too
    """

    embed_text = None
    message_text = ""
    created = None

    def __init__(self,
                 *items: discord.ui.Item,
                 timeout: Optional[float] = 60*60,  # 24h from last click
                 embed: Optional[Embed] = None,
                 message: Optional[str] = None,
                 bot=None
                 ):
        if embed:
            if isinstance(embed, dict):
                self.embed_text = Embed.from_dict(embed)
            elif isinstance(embed, Embed):
                self.embed_text = embed
        if message:
            self.message_text = message
        self.bot = bot
        self.created = timezone.now()

        super().__init__(*items, timeout=timeout)

    # a custom_id must be set
    @discord.ui.select(placeholder="Who do you need help from?", options=get_groups())
    async def select_callback(self, select, interaction):
        sup_channel = models.TicketGroups.get_solo().ticket_channel.channel
        ch = interaction.guild.get_channel(sup_channel)
        grp = discord.utils.get(interaction.guild.roles, name=select.values[0])
        th = await ch.create_thread(name=f"{interaction.user.display_name} | {select.values[0]} | {timezone.now().strftime('%Y-%m-%d %H:%M')}",
                                    # message=f"Ping in here if your request is urgent <@{interaction.user.id}>, Someone from <#{grp.id}> will be here soon!",
                                    auto_archive_duration=10080,
                                    type=discord.ChannelType.private_thread,
                                    reason=None)
        msg = f"<@{interaction.user.id}> needs help!, Someone from <@&{grp.id}> will get in touch soon!"
        embd = Embed(title="Private Thread Guide",
                     description="To add a person to this thread simply `@ping` them. This works with `@groups` as well to bulk add people to the channel. Use wisely, abuse will not be tolerated.\n\nThis is a beta feature if you experience issues please contact the admins. :heart:")
        await th.send(msg, embed=embd)
        await interaction.response.edit_message(content="Ping in the thread created for urgent help!", view=None)

    async def on_timeout(self) -> None:
        await super().on_timeout()


class HelpCog(commands.Cog):
    """
        Compliance Related thingies
    """

    def __init__(self, bot):
        self.bot = bot

    @command(name='help', guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def slash_halp(
        self,
        ctx,
    ):
        """
            Help me authbot i dont know who to ping!
        """
        await ctx.defer(ephemeral=True)
        return await ctx.respond(view=HelpView())

    @command(name='close_ticket', guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def slash_close(
        self,
        ctx,
    ):
        """
            Mark help thread as completed and archive it!
        """
        ch = ctx.channel
        if ch.type != ChannelType.private_thread:
            return await ctx.respond("Not a private thread!", ephemeral=True)
        await ctx.defer()
        embd = Embed(title="Thread Marked Complete",
                     description=f"{ctx.user.display_name} has marked this thread as completed. To reopen simply start chatting again.")
        await ctx.respond(embed=embd)
        return await ch.archive()


def setup(bot):
    bot.add_cog(HelpCog(bot))



import logging

from discord.colour import Color
from discord.commands import SlashCommandGroup
from discord.embeds import Embed
from discord.ext import commands

from django.conf import settings

from aadiscordbot import __branch__, __version__
from aadiscordbot.app_settings import get_site_url

logger = logging.getLogger(__name__)


class About(commands.Cog):
    """
    All about me!
    """

    def __init__(self, bot):
        self.bot = bot

    about_commands = SlashCommandGroup("about", "All about the Bot and Auth", guild_ids=[
                                       int(settings.DISCORD_GUILD_ID)])

    @about_commands.command(name="discordbot", description="About the Discord Bot", guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def discordbot(self, ctx):
        """
        All about the bot
        """
        embed = Embed(title="AuthBot: The Authening")
        embed.set_thumbnail(
            url="https://cdn.discordapp.com/icons/516758158748811264/ae3991584b0f800b181c936cfc707880.webp?size=128"
        )
        embed.colour = Color.blue()

        embed.description = "This is a multi-de-functional discord bot tailored specifically for Alliance Auth Shenanigans."
        regex = r"^(.+)\/d.+"

        embed.set_footer(
            text="Lovingly developed for Init.™ by AaronRin and ArielKable")

        embed.add_field(
            name="Number of Servers:", value=len(self.bot.guilds), inline=True
        )
        members = 0
        for g in self.bot.guilds:
            members += g.member_count
        embed.add_field(name="Unwilling Monitorees:",
                        value=members, inline=True)
        embed.add_field(
            name="Auth Link", value=get_site_url(), inline=False
        )
        embed.add_field(
            name="Version", value=f"{__version__}@{__branch__}", inline=False
        )

        return await ctx.respond(embed=embed)

    @about_commands.command(name="server", description="About this Discord Server", guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def server(self, ctx):
        """
        All about a server
        """
        embed = Embed(title=ctx.guild.name)

        if ctx.guild.icon:
            embed.set_thumbnail(
                url=ctx.guild.icon.url
            )
        embed.color = Color.blue()
        embed.description = "Alliance Auth Managed EvE Online Discord Server!"
        if ctx.guild.description:
            embed.description = ctx.guild.description
        embed.set_footer(
            text="AuthBot Lovingly developed for Init.™ by AaronRin and ArielKable")

        members = ctx.guild.member_count
        embed.add_field(name="Unwilling Monitorees:",
                        value=members, inline=True)

        channels = len(ctx.guild.channels)
        cats = len(ctx.guild.categories)
        embed.add_field(name="Channel Count:",
                        value=channels-cats, inline=True)

        roles = len(ctx.guild.roles)
        embed.add_field(name="Role Count:",
                        value=roles, inline=True)

        embed.add_field(
            name="Auth Link", value=get_site_url(), inline=False
        )

        return await ctx.respond(embed=embed)


def setup(bot):
    bot.add_cog(About(bot))



# Generated by Django 3.2.8 on 2022-01-04 08:04

from django.db import migrations, models


def add_settings(apps, schema_editor):
    AuthBotConfiguration = apps.get_model(
        'aadiscordbot', 'AuthBotConfiguration')

    # StructureLostShields
    AuthBotConfiguration.objects.create()


class Migration(migrations.Migration):

    dependencies = [
        ('aadiscordbot', '0003_authbotconfiguration'),
    ]

    operations = [
        migrations.RunPython(add_settings)
    ]



import logging

from celery import shared_task
from discord import Embed

import django
import django.db
from django.conf import settings
from django.contrib.auth.models import User

from allianceauth.services.modules.discord.models import DiscordUser

logger = logging.getLogger(__name__)


def send_message(message="", channel_id: int = None, user_id: int = None, user: User = None, user_pk: User = None, embed: Embed = None, countdown: int = 0):
    ''' Helper function to queue discord messages from the bot

        :param message: (optional) The text to send (default "").
        :param channel_id: (optional) the discord channel_id to send a message to.
        :param user_id: (optional) the discord user_id to send a message to as a DM.
        :param user: (optional) the Auth User Model to send a message as a DM.
        :param user_pk: (optional) the Auth Users PK to send a message as a DM.
        :param embed: (optional) the embed to send.
        :param countdown: (optional) seconds to delay the message send.
    '''
    if embed:
        embed = embed.to_dict()

    if channel_id:
        send_channel_message_by_discord_id.apply_async(
            args=[channel_id, message],
            kwargs={"embed": embed},
            countdown=countdown)

    if user_id:
        send_direct_message_by_discord_id.apply_async(
            args=[user_id, message],
            kwargs={"embed": embed},
            countdown=countdown)

    if user:
        pk = user.pk
        send_direct_message_by_user_id.apply_async(
            args=[pk, message],
            kwargs={"embed": embed},
            countdown=countdown)
    elif user_pk:
        send_direct_message_by_user_id.apply_async(
            args=[user_pk, message],
            kwargs={"embed": embed},
            countdown=countdown)

# Note these Tasks do not DO anything. They can simply be called by AA to add the tasks to our Queue of choice to be consumed by bot.queueconsumer


@shared_task
def send_channel_message_by_discord_id(channel_id, message_content, embed=None, view_class: str = None, view_args: list = [], view_kwargs: dict = {}):
    # Queue a message to a Discord Channel
    raise Exception(
        f"This function should be called asynchronously. Failed to queue a message to Channel {channel_id}")


@shared_task
def send_channel_message(channel_id, message_content, embed=False):
    # DEPRECATED shim to queue a message to a Discord Channel
    raise Exception(
        f"This function should be called asynchronously. Failed to queue a message to Channel {channel_id}. Warning! This function is deprecated.")


@shared_task
def send_direct_message_by_discord_id(discord_user_id, message_content, embed=False):
    # Queue a Private Message to a specific user
    raise Exception(
        f"This function should be called asynchronously. Failed to queue a message to User {discord_user_id}")


@shared_task
def send_direct_message(user_id, message_content, embed=False):
    # DEPRECATED shim to queue a Private Message to a specific user
    raise Exception(
        f"This function should be called asynchronously. Failed to queue a message to User {discord_user_id}. Warning! This function is deprecated.")


@shared_task
def send_direct_message_by_user_id(user_pk, message_content, embed=False):
    # Queue a Private Message to a specific user
    raise Exception(
        f"This function should be called asynchronously. Failed to queue a message to User {user_pk}")


@shared_task
def pop_user_group_cache(user_pk):
    raise Exception(
        f"This function should be called asynchronously. Failed to queue...")



import datetime
import logging

import pendulum
from discord import Option
from discord.colour import Color
from discord.commands import SlashCommandGroup
from discord.embeds import Embed
from discord.ext import commands

from django.conf import settings
from django.utils import timezone

from allianceauth.services.modules.discord.models import DiscordUser
from esi.models import Token

from aadiscordbot import app_settings, providers

logger = logging.getLogger(__name__)


class Sov(commands.Cog):
    """
    All about sov!
    """

    def __init__(self, bot):
        self.bot = bot

    sov_commands = SlashCommandGroup("sov", "Commands for Managing/Attacking Sov", guild_ids=[
        int(settings.DISCORD_GUILD_ID)])

    def get_search_token(self, uid):
        user = DiscordUser.objects.get(uid=uid).user
        tokens = Token.objects.filter(user=user).require_scopes(
            ['esi-search.search_structures.v1'])
        return tokens.first()

    @sov_commands.command(name='lowadm', guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def lowadm(self, ctx, adm: Option(float, description="Optional ADM Level to flag under.", required=False)):
        """
        Systems with low ADMs.
        """
        if not adm:
            adm = app_settings.get_low_adm()
        #    @commands.command(pass_context=True)
        #    @message_in_channels(settings.SOV_DISCORD_BOT_CHANNELS)

        if ctx.channel.id not in settings.SOV_DISCORD_BOT_CHANNELS:
            return await ctx.respond(f"You do not have permission to use this command here.", ephemeral=True)

        await ctx.defer()

        own_ids = settings.DISCORD_BOT_SOV_STRUCTURE_OWNER_IDS

        include_regions = settings.DISCORD_BOT_ADM_REGIONS
        include_systems = settings.DISCORD_BOT_ADM_SYSTEMS
        include_constel = settings.DISCORD_BOT_ADM_CONSTELLATIONS

        sov_structures = providers.esi.client.Sovereignty.get_sovereignty_structures().result()

        names = {}
        for s in sov_structures:
            if s.get('alliance_id') in own_ids:
                if s.get('vulnerability_occupancy_level'):
                    if s.get('vulnerability_occupancy_level') < adm:
                        names[s.get('solar_system_id')] = {
                            "system_name": s.get('solar_system_id'),
                            "adm": s.get('vulnerability_occupancy_level')
                        }

        if len(names) == 0:
            await ctx.respond(f"All above {adm} :ok_hand:")
            return True

        systems = [k for k, v in names.items()]
        constelations = {}
        for n in systems:
            system = providers.esi.client.Universe.get_universe_systems_system_id(
                system_id=n).result()
            names[n]["system_name"] = system.get("name")
            names[n]["system_id"] = system.get("system_id")
            names[n]["constellation_id"] = system.get("constellation_id")
            if system.get("constellation_id") not in constelations:
                constelations[system.get("constellation_id")] = {}

        for c, v in constelations.items():
            const = providers.esi.client.Universe.get_universe_constellations_constellation_id(
                constellation_id=c).result()
            region = providers.esi.client.Universe.get_universe_regions_region_id(
                region_id=const.get("region_id")).result()
            v["region_id"] = const.get("region_id")
            v["region_name"] = region.get("name")
            v["constellation_name"] = const.get("name")

        out_array = {}
        for k, v in names.items():
            out_array[k] = {**v, **constelations[v["constellation_id"]]}

        output = {}
        base_str = "**{}** ADM:{}"
        urls = {}
        for k, h in sorted(out_array.items(), key=lambda e: e[1]['adm']):
            show = False
            if h['region_id'] in include_regions:
                show = True
            elif h['constellation_id'] in include_constel:
                show = True
            elif h['system_id'] in include_systems:
                show = True
            if show:
                if h['region_name'] not in output:
                    output[h['region_name']] = []
                output[h['region_name']].append(
                    base_str.format(
                        h['system_name'],
                        h['adm']
                    )
                )
                if h['region_name'].replace(" ", "_") not in urls:
                    urls[h['region_name'].replace(" ", "_")] = []
                urls[h['region_name'].replace(" ", "_")].append(
                    h['system_name'].replace(" ", "_")
                )
        url = "https://evemaps.dotlan.net/map/{}/{}#adm"

        if len(output) > 0:
            embed = Embed(title="Low ADMs!")
            embed.set_thumbnail(
                url="https://avatars3.githubusercontent.com/u/39349660?s=200&v=4"
            )
            embed.colour = Color.red()
            embed.description = f"Showing systems with ADMs below {adm}. Due to an ESI bug this data might only update once a day at around 2200-2300 Eve Time. **YMMY**\n[For more info please see this issue](https://github.com/esi/esi-issues/issues/1092)"

            await ctx.respond(embed=embed)

            for k, v in output.items():
                n = 25
                chunks = [list(v[i * n:(i + 1) * n])
                          for i in range((len(v) + n - 1) // n)]
                for chunk in chunks:
                    await ctx.send("__{}__\n{}".format(k, "\n".join(chunk)))
            _urls = []
            for r, s in urls.items():
                _urls.append(url.format(r, ",".join(s)))
            await ctx.send("\n\n".join(_urls))
        else:
            await ctx.respond(f"No Systems with ADM below {adm}!")

        return True

    @sov_commands.command(name='vulnerable', guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def vuln(self, ctx, name_search: Option(str, description="String to search against the sov database.")):
        """
        Vulnerable Sov Structures for region/constelation/system/alliance
        """
        if ctx.channel.id not in settings.SOV_DISCORD_BOT_CHANNELS:
            return await ctx.respond(f"You do not have permission to use this command here.", ephemeral=True)

        await ctx.defer()

        token = self.get_search_token(ctx.author.id)
        if not token:
            return await ctx.respond("No Search token found on auth, please add one", ephemeral=True)
        name_ids = providers.esi.client.Search.get_characters_character_id_search(
            categories=['constellation',
                        'solar_system',
                        'region',
                        'alliance'
                        ],
            search=name_search,
            character_id=token.character_id,
            token=token.valid_access_token()
        ).result()

        hit_ids = {
            "a": name_ids.get("alliance") or [],
            "c": name_ids.get("constellation") or [],
            "s": name_ids.get("solar_system") or [],
            "r": name_ids.get("region") or [],
        }

        for r in hit_ids['r']:
            constellations = providers.esi.client.Universe.get_universe_regions_region_id(
                region_id=r).result()["constellations"]
            for c in constellations:
                if c not in hit_ids["c"]:
                    hit_ids["c"].append(c)

        for c in hit_ids['c']:
            systems = providers.esi.client.Universe.get_universe_constellations_constellation_id(
                constellation_id=c).result()["systems"]
            for s in systems:
                if s not in hit_ids["s"]:
                    hit_ids["s"].append(s)

        sov_structures = providers.esi.client.Sovereignty.get_sovereignty_structures().result()

        hits = []
        names = []
        alliances = []
        dt_comp = datetime.datetime.utcnow().replace(tzinfo=timezone.utc) + \
            datetime.timedelta(hours=1)

        for s in sov_structures:
            start = s.get('vulnerable_start_time', False)
            if start:
                if start < dt_comp:
                    if s.get('solar_system_id') in hit_ids["s"] or s.get('alliance_id') in hit_ids["a"]:
                        alliances.append(s.get('alliance_id'))
                        names.append(s.get('solar_system_id'))
                        names.append(s.get('structure_type_id'))
                        hits.append(s)

        if len(names) == 0:
            await ctx.respond(f":sad: Nothing found for '{name_search}'")
            return True

        names_alli = {}
        for a in set(alliances):
            res = providers.esi.client.Alliance.get_alliances_alliance_id(
                alliance_id=a).result()
            names_alli[a] = res.get("ticker")

        names = providers.esi.client.Universe.post_universe_names(
            ids=list(set(names))).result()

        nms = {}
        for n in names:
            nms[n.get("id")] = n.get("name")

        for hit in hits:
            hit['system_name'] = nms[hit.get('solar_system_id')]
            if hit.get("structure_type_id") == 32226:
                hit['structure'] = "TCU"
            elif hit.get("structure_type_id") == 32458:
                hit['structure'] = "IHUB"
            else:
                hit['structure'] = "¯\\_(ツ)_/¯"

            hit['alliance_name'] = names_alli[hit.get('alliance_id')]

        output = []
        base_str = "**{}** {} (ADM {})[**{}**] Vulnerable{}"
        dt_now = pendulum.now(tz="UTC")
        for h in sorted(hits, key=lambda k: k['vulnerable_start_time']):
            time = ""
            if h['vulnerable_start_time'] > dt_now:
                time = " in **{}**".format(pendulum.now(tz="UTC").diff_for_humans(
                    h['vulnerable_start_time'], absolute=True
                ))
            else:
                time = " for **{}**".format(pendulum.now(tz="UTC").diff_for_humans(
                    h['vulnerable_end_time'], absolute=True
                ))
            output.append(
                base_str.format(
                    h['system_name'],
                    h['structure'],
                    h['vulnerability_occupancy_level'],
                    h['alliance_name'],
                    time
                )
            )

        n = 10
        chunks = [list(output[i * n:(i + 1) * n])
                  for i in range((len(output) + n - 1) // n)]
        overflow = ""
        if len(output) > 50:
            overflow = "Only showing first 50..."
        await ctx.respond("Found {} Vunerable Structures for `{}`\n{}\n".format(
            len(output),
            name_search,
            overflow
        ), ephemeral=False
        )

        for c in chunks[:5]:
            await ctx.send("\n".join(c))

    @sov_commands.command(name='search', guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def sov(self, ctx, name_search: Option(str, description="String to search against the sov database.")):
        """
        Sov Details for region/constelation/system/alliance
        """
        if ctx.channel.id not in settings.SOV_DISCORD_BOT_CHANNELS:
            return await ctx.respond(f"You do not have permission to use this command here.", ephemeral=True)

        await ctx.defer()
        token = self.get_search_token(ctx.author.id)
        if not token:
            return await ctx.respond("No Search token found on auth, please add one", ephemeral=True)
        name_ids = providers.esi.client.Search.get_characters_character_id_search(
            categories=['constellation',
                        'solar_system',
                        'region',
                        'alliance'
                        ],
            search=name_search,
            character_id=token.character_id,
            token=token.valid_access_token()
        ).result()

        hit_ids = {
            "a": name_ids.get("alliance") or [],
            "c": name_ids.get("constellation") or [],
            "s": name_ids.get("solar_system") or [],
            "r": name_ids.get("region") or [],
        }

        for r in hit_ids['r']:
            constellations = providers.esi.client.Universe.get_universe_regions_region_id(
                region_id=r).result()["constellations"]
            for c in constellations:
                if c not in hit_ids["c"]:
                    hit_ids["c"].append(c)

        for c in hit_ids['c']:
            systems = providers.esi.client.Universe.get_universe_constellations_constellation_id(
                constellation_id=c).result()["systems"]
            for s in systems:
                if s not in hit_ids["s"]:
                    hit_ids["s"].append(s)

        sov_structures = providers.esi.client.Sovereignty.get_sovereignty_structures().result()

        hits = []
        names = []
        alliances = []

        for s in sov_structures:
            start = s.get('vulnerable_start_time', False)
            if start:
                if s.get('solar_system_id') in hit_ids["s"] or s.get('alliance_id') in hit_ids["a"]:
                    alliances.append(s.get('alliance_id'))
                    names.append(s.get('solar_system_id'))
                    names.append(s.get('structure_type_id'))
                    hits.append(s)

        if len(names) == 0:
            await ctx.respond(f":sad: Nothing found for '{name_search}'")
            return True

        names_alli = {}
        for a in set(alliances):
            res = providers.esi.client.Alliance.get_alliances_alliance_id(
                alliance_id=a).result()
            names_alli[a] = res.get("ticker")

        names = providers.esi.client.Universe.post_universe_names(
            ids=list(set(names))).result()

        nms = {}
        for n in names:
            nms[n.get("id")] = n.get("name")

        for hit in hits:
            hit['system_name'] = nms[hit.get('solar_system_id')]
            if hit.get("structure_type_id") == 32226:
                hit['structure'] = "TCU"
            elif hit.get("structure_type_id") == 32458:
                hit['structure'] = "IHUB"
            else:
                hit['structure'] = "¯\\_(ツ)_/¯"

            hit['alliance_name'] = names_alli[hit.get('alliance_id')]

        output = []
        base_str = "**{}** {} (ADM {})[**{}**] Vulnerable{}"
        dt_now = pendulum.now(tz="UTC")
        for h in sorted(hits, key=lambda k: k['vulnerable_start_time']):
            time = ""
            time = " for **{}**".format(pendulum.now(tz="UTC").diff_for_humans(
                h['vulnerable_end_time'], absolute=True
            ))

            if h['vulnerable_start_time']:
                if h['vulnerable_start_time'] > dt_now:
                    time = " in **{}**".format(pendulum.now(tz="UTC").diff_for_humans(
                        h['vulnerable_start_time'], absolute=True
                    ))
            output.append(
                base_str.format(
                    h['system_name'],
                    h['structure'],
                    h['vulnerability_occupancy_level'],
                    h['alliance_name'],
                    time
                )
            )

        n = 15
        chunks = [list(output[i * n:(i + 1) * n])
                  for i in range((len(output) + n - 1) // n)]
        overflow = ""
        if len(output) > 50:
            overflow = "Only showing first 50..."

        await ctx.respond("Found {} Sov structures for `{}`\n{}\n".format(
            len(output),
            name_search,
            overflow
        ), ephemeral=False
        )

        for c in chunks[:5]:
            await ctx.send("\n".join(c))


def setup(bot):
    bot.add_cog(Sov(bot))



import logging

from discord import User
from discord.ext import commands

from django.conf import settings

logger = logging.getLogger(__name__)


class EasterEggs(commands.Cog):
    """
    Stupid commands that don't belong anywhere
    These will not appear in Help menus
    Have limited to no real use
    I was bored
    """

    def __init__(self, bot):
        self.bot = bot

    @commands.slash_command(name='happybirthday', guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def happybirthday(self, ctx,  user: User):
        """
        Takes one Discord User as an argument, Wishes this user a happy birthday
        If no user is passed, responds to the context user
        "Useful" to verify the bot is alive and functioning
        """
        await ctx.trigger_typing()
        return await ctx.respond(f"Happy Birthday {user.mention}")


def setup(bot):
    bot.add_cog(EasterEggs(bot))
